@@ -26,7 +26,8 @@ branches and make a code block out of it. But it doesn't contain any target
 language syntax.
 """
 
-from nuitka.build.DataComposerInterface import deriveModuleConstantsBlobName
+from nuitka.plugins.Plugins import Plugins
+from nuitka.utils.CStrings import encodePythonStringToC
 
 from . import Contexts
 from .AsyncgenCodes import (
@@ -72,7 +73,7 @@ from .BuiltinCodes import (
     generateBuiltinXrange2Code,
     generateBuiltinXrange3Code,
 )
-from .CallCodes import generateCallCode, getCallsCode, getCallsDecls
+from .CallCodes import generateCallCode, getCallsCode
 from .ClassCodes import generateBuiltinSuperCode, generateSelectMetaclassCode
 from .CodeHelpers import setExpressionDispatchDict, setStatementDispatchDict
 from .ComparisonCodes import (
@@ -86,11 +87,8 @@ from .ConditionalCodes import (
     generateConditionalCode,
 )
 from .ConstantCodes import (
-    generateConstantEllipsisReferenceCode,
-    generateConstantFalseReferenceCode,
-    generateConstantNoneReferenceCode,
+    generateConstantGenericAliasCode,
     generateConstantReferenceCode,
-    generateConstantTrueReferenceCode,
     getConstantsDefinitionCode,
 )
 from .CoroutineCodes import (
@@ -469,15 +467,15 @@ def generateModuleCode(module, data_filename):
         module=module,
         function_decl_codes=function_decl_codes,
         function_body_codes=function_body_codes,
-        module_const_blob_name=deriveModuleConstantsBlobName(data_filename),
+        module_const_blob_name=encodePythonStringToC(
+            Plugins.deriveModuleConstantsBlobName(data_filename)
+        ),
         context=context,
     )
 
 
 def generateHelpersCode():
-    calls_decl_code = getCallsDecls()
-
-    calls_body_code = getCallsCode()
+    calls_decl_code, calls_body_code = getCallsCode()
 
     constants_header_code, constants_body_code = getConstantsDefinitionCode()
 
@@ -577,9 +575,9 @@ setExpressionDispatchDict(
         "EXPRESSION_CALL_KEYWORDS_ONLY": generateCallCode,
         "EXPRESSION_CALL_NO_KEYWORDS": generateCallCode,
         "EXPRESSION_CALL": generateCallCode,
-        "EXPRESSION_CONSTANT_NONE_REF": generateConstantNoneReferenceCode,
-        "EXPRESSION_CONSTANT_TRUE_REF": generateConstantTrueReferenceCode,
-        "EXPRESSION_CONSTANT_FALSE_REF": generateConstantFalseReferenceCode,
+        "EXPRESSION_CONSTANT_NONE_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TRUE_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_FALSE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_STR_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_STR_EMPTY_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_UNICODE_REF": generateConstantReferenceCode,
@@ -590,7 +588,7 @@ setExpressionDispatchDict(
         "EXPRESSION_CONSTANT_LONG_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_FLOAT_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_COMPLEX_REF": generateConstantReferenceCode,
-        "EXPRESSION_CONSTANT_ELLIPSIS_REF": generateConstantEllipsisReferenceCode,
+        "EXPRESSION_CONSTANT_ELLIPSIS_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_DICT_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_DICT_EMPTY_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_TUPLE_REF": generateConstantReferenceCode,
@@ -605,7 +603,9 @@ setExpressionDispatchDict(
         "EXPRESSION_CONSTANT_SLICE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_XRANGE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_TYPE_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TYPE_SUBSCRIPTABLE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_BYTEARRAY_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_GENERIC_ALIAS": generateConstantGenericAliasCode,
         "EXPRESSION_CONDITIONAL": generateConditionalCode,
         "EXPRESSION_CONDITIONAL_OR": generateConditionalAndOrCode,
         "EXPRESSION_CONDITIONAL_AND": generateConditionalAndOrCode,
