@@ -79,10 +79,10 @@ from nuitka.utils.SharedLibraries import (
     getPyWin32Dir,
     getSharedLibraryRPATH,
     getWindowsDLLVersion,
-    removeMacOSCodeSignature,
     removeSharedLibraryRPATH,
     removeSxsFromDLL,
 )
+from nuitka.utils.Signing import removeMacOSCodeSignature
 from nuitka.utils.ThreadedExecutor import ThreadPoolExecutor, waitWorkers
 from nuitka.utils.Timing import TimerReport
 
@@ -165,11 +165,6 @@ __file__ = (__nuitka_binary_dir + '%s%s') if '__nuitka_binary_dir' in dict(__bui
             "PREFIXES = [sys.prefix, sys.exec_prefix]", "PREFIXES = []"
         )
 
-        # Anaconda3 4.1.2 site.py
-        source_code = source_code.replace(
-            "def main():", "def main():return\n\nif 0:\n def _unused():"
-        )
-
     if Options.isShowInclusion():
         inclusion_logger.info(
             "Freezing module '%s' (from '%s')." % (module_name, filename)
@@ -298,7 +293,7 @@ print("\\n".join(sorted(
             origin = parts[1].split()[0]
 
             if python_version >= 0x300:
-                module_name = module_name.decode("utf-8")
+                module_name = module_name.decode("utf8")
 
             module_name = ModuleName(module_name)
 
@@ -307,17 +302,30 @@ print("\\n".join(sorted(
                 # chance to do anything, we need to preserve it.
                 filename = parts[1][len(b"precompiled from ") :]
                 if python_version >= 0x300:
-                    filename = filename.decode("utf-8")
+                    filename = filename.decode("utf8")
 
                 # Do not leave standard library when freezing.
                 if not isStandardLibraryPath(filename):
                     continue
 
                 detections.append((module_name, 3, "precompiled", filename))
+            elif origin == b"from" and python_version < 0x300:
+                filename = parts[1][len(b"from ") :]
+                if python_version >= 0x300:  # For consistency, and maybe later reuse
+                    filename = filename.decode("utf8")
+
+                # Do not leave standard library when freezing.
+                if not isStandardLibraryPath(filename):
+                    continue
+
+                if filename.endswith(".py"):
+                    detections.append((module_name, 2, "sourcefile", filename))
+                else:
+                    assert False
             elif origin == b"sourcefile":
                 filename = parts[1][len(b"sourcefile ") :]
                 if python_version >= 0x300:
-                    filename = filename.decode("utf-8")
+                    filename = filename.decode("utf8")
 
                 # Do not leave standard library when freezing.
                 if not isStandardLibraryPath(filename):
@@ -341,7 +349,7 @@ print("\\n".join(sorted(
                 # or self compiled Python installations.
                 filename = parts[1][len(b"dynamically loaded from ") :]
                 if python_version >= 0x300:
-                    filename = filename.decode("utf-8")
+                    filename = filename.decode("utf8")
 
                 # Do not leave standard library when freezing.
                 if not isStandardLibraryPath(filename):
@@ -636,9 +644,24 @@ _linux_dll_ignore_list = (
     # libraries like MESA to not load any drivers, so we exclude it too, and
     # it can be assumed to be installed everywhere anyway.
     "libstdc++.so.",
+    # The DRM layer should also be taken from the OS in question and won't
+    # allow loading native drivers otherwise.
+    "libdrm.so",
 )
 
 
+def _getLdLibraryPath(package_name, python_rpath, original_dir):
+    ld_library_path = OrderedSet()
+    if python_rpath:
+        ld_library_path.add(python_rpath)
+    ld_library_path.update(getPackageSpecificDLLDirectories(package_name))
+
+    if original_dir is not None:
+        ld_library_path.add(original_dir)
+
+    return ld_library_path
+
+
 def _detectBinaryPathDLLsPosix(dll_filename, package_name, original_dir):
     # This is complex, as it also includes the caching mechanism
     # pylint: disable=too-many-branches
@@ -648,7 +671,6 @@ def _detectBinaryPathDLLsPosix(dll_filename, package_name, original_dir):
 
     # Ask "ldd" about the libraries being used by the created binary, these
     # are the ones that interest us.
-    result = set()
 
     # This is the rpath of the Python binary, which will be effective when
     # loading the other DLLs too. This happens at least for Python installs
@@ -662,16 +684,14 @@ def _detectBinaryPathDLLsPosix(dll_filename, package_name, original_dir):
                 "$ORIGIN", os.path.dirname(sys.executable)
             )
 
-    ld_library_path = OrderedSet()
-    if _detected_python_rpath:
-        ld_library_path.add(_detected_python_rpath)
-    ld_library_path.update(getPackageSpecificDLLDirectories(package_name))
-
-    if original_dir is not None:
-        ld_library_path.add(original_dir)
-        # ld_library_path.update(getSubDirectories(original_dir, ignore_dirs=("__pycache__",)))
-
-    with withEnvironmentPathAdded("LD_LIBRARY_PATH", *ld_library_path):
+    with withEnvironmentPathAdded(
+        "LD_LIBRARY_PATH",
+        *_getLdLibraryPath(
+            package_name=package_name,
+            python_rpath=_detected_python_rpath,
+            original_dir=original_dir,
+        )
+    ):
         process = subprocess.Popen(
             args=["ldd", dll_filename],
             stdin=getNullInput(),
@@ -681,48 +701,50 @@ def _detectBinaryPathDLLsPosix(dll_filename, package_name, original_dir):
 
         stdout, stderr = process.communicate()
 
-        stderr = b"\n".join(
-            line
-            for line in stderr.splitlines()
-            if not line.startswith(
-                b"ldd: warning: you do not have execution permission for"
-            )
+    stderr = b"\n".join(
+        line
+        for line in stderr.splitlines()
+        if not line.startswith(
+            b"ldd: warning: you do not have execution permission for"
         )
+    )
 
-        inclusion_logger.debug("ldd output for %s is:\n%s" % (dll_filename, stdout))
+    inclusion_logger.debug("ldd output for %s is:\n%s" % (dll_filename, stdout))
 
-        if stderr:
-            inclusion_logger.debug("ldd error for %s is:\n%s" % (dll_filename, stderr))
+    if stderr:
+        inclusion_logger.debug("ldd error for %s is:\n%s" % (dll_filename, stderr))
 
-        for line in stdout.split(b"\n"):
-            if not line:
-                continue
+    result = set()
 
-            if b"=>" not in line:
-                continue
+    for line in stdout.split(b"\n"):
+        if not line:
+            continue
 
-            part = line.split(b" => ", 2)[1]
+        if b"=>" not in line:
+            continue
 
-            if b"(" in part:
-                filename = part[: part.rfind(b"(") - 1]
-            else:
-                filename = part
+        part = line.split(b" => ", 2)[1]
 
-            if not filename:
-                continue
+        if b"(" in part:
+            filename = part[: part.rfind(b"(") - 1]
+        else:
+            filename = part
 
-            if python_version >= 0x300:
-                filename = filename.decode("utf-8")
+        if not filename:
+            continue
 
-            # Sometimes might use stuff not found or supplied by ldd itself.
-            if filename in ("not found", "ldd"):
-                continue
+        if python_version >= 0x300:
+            filename = filename.decode("utf8")
 
-            # Do not include kernel DLLs on the ignore list.
-            if os.path.basename(filename).startswith(_linux_dll_ignore_list):
-                continue
+        # Sometimes might use stuff not found or supplied by ldd itself.
+        if filename in ("not found", "ldd"):
+            continue
 
-            result.add(filename)
+        # Do not include kernel DLLs on the ignore list.
+        if os.path.basename(filename).startswith(_linux_dll_ignore_list):
+            continue
+
+        result.add(filename)
 
     ldd_result_cache[dll_filename] = result
 
@@ -740,22 +762,34 @@ def _detectBinaryPathDLLsPosix(dll_filename, package_name, original_dir):
     return sub_result
 
 
-def _detectBinaryPathDLLsMacOS(original_dir, binary_filename, keep_unresolved):
-    result = OrderedSet()
-
-    process = subprocess.Popen(
-        args=["otool", "-L", binary_filename],
-        stdin=getNullInput(),
-        stdout=subprocess.PIPE,
-        stderr=subprocess.PIPE,
-    )
+def _detectBinaryPathDLLsMacOS(
+    original_dir, binary_filename, package_name, keep_unresolved
+):
+    with withEnvironmentPathAdded(
+        "DYLD_LIBRARY_PATH",
+        *_getLdLibraryPath(
+            package_name=package_name, python_rpath=None, original_dir=original_dir
+        )
+    ):
+        process = subprocess.Popen(
+            args=["otool", "-L", binary_filename],
+            stdin=getNullInput(),
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+        )
 
     stdout, _stderr = process.communicate()
-    system_paths = ("/usr/lib/", "/System/Library/Frameworks/")
+    system_paths = (
+        "/usr/lib/",
+        "/System/Library/Frameworks/",
+        "/System/Library/PrivateFrameworks/",
+    )
+
+    result = OrderedSet()
 
     for line in stdout.split(b"\n")[1:]:
         if str is not bytes:
-            line = line.decode("utf-8")
+            line = line.decode("utf8")
 
         if not line:
             continue
@@ -769,7 +803,10 @@ def _detectBinaryPathDLLsMacOS(original_dir, binary_filename, keep_unresolved):
             result.add(filename)
 
     resolved_result = _resolveBinaryPathDLLsMacOS(
-        original_dir, binary_filename, result, keep_unresolved
+        original_dir=original_dir,
+        binary_filename=binary_filename,
+        paths=result,
+        keep_unresolved=keep_unresolved,
     )
     return resolved_result
 
@@ -820,7 +857,7 @@ def _detectBinaryRPathsMacOS(original_dir, binary_filename):
         if o.endswith(b"cmd LC_RPATH"):
             line = lines[i + 2]
             if str is not bytes:
-                line = line.decode("utf-8")
+                line = line.decode("utf8")
 
             line = line.split("path ", 1)[1]
             line = line.split(" (offset", 1)[0]
@@ -1023,7 +1060,7 @@ def detectBinaryDLLs(
         )
     elif Utils.isWin32Windows():
         with TimerReport(
-            message="Running depends.exe for %s took %%.2f seconds" % binary_filename,
+            message="Running 'depends.exe' for %s took %%.2f seconds" % binary_filename,
             decider=Options.isShowProgress,
         ):
             return detectBinaryPathDLLsWindowsDependencyWalker(
@@ -1039,6 +1076,7 @@ def detectBinaryDLLs(
         return _detectBinaryPathDLLsMacOS(
             original_dir=os.path.dirname(original_filename),
             binary_filename=original_filename,
+            package_name=package_name,
             keep_unresolved=False,
         )
     else:
@@ -1120,6 +1158,7 @@ def fixupBinaryDLLPathsMacOS(binary_filename, dll_map, original_location):
     rpath_map = _detectBinaryPathDLLsMacOS(
         original_dir=os.path.dirname(original_location),
         binary_filename=original_location,
+        package_name=None,
         keep_unresolved=True,
     )
     for i, o in enumerate(dll_map):
