@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -131,9 +131,6 @@ class StatementAssignmentVariable(StatementChildHavingBase):
     it can be determined.
     """
 
-    # Using slots, they don't need that
-    # pylint: disable=access-member-before-definition,attribute-defined-outside-init
-
     kind = "STATEMENT_ASSIGNMENT_VARIABLE"
 
     named_child = "source"
@@ -250,7 +247,7 @@ class StatementAssignmentVariable(StatementChildHavingBase):
             # sequence and compute that instead.
             statements = [
                 makeStatementExpressionOnlyReplacementNode(side_effect, self)
-                for side_effect in source.getSideEffects()
+                for side_effect in source.subnode_side_effects
             ]
 
             statements.append(self)
@@ -260,7 +257,7 @@ class StatementAssignmentVariable(StatementChildHavingBase):
 
             # Need to update ourselves to no longer reference the side effects,
             # but go to the wrapped thing.
-            self.subnode_source = source.getExpression()
+            self.setChild("source", source.subnode_expression)
 
             result = makeStatementsSequenceReplacementNode(
                 statements=statements, node=self
@@ -349,8 +346,13 @@ Removed assignment of %s from itself which is known to be defined."""
 
             if last_trace is not None and not last_trace.getMergeOrNameUsageCount():
                 if source.isCompileTimeConstant():
-                    # TODO: We do not trust these yet
-                    if not variable.isModuleVariable():
+                    if (
+                        variable.isModuleVariable()
+                        or variable.owner.locals_scope.isUnoptimizedFunctionScope()
+                    ):
+                        # TODO: We do not trust these yet a lot, but more might be
+                        pass
+                    else:
                         # Unused constants can be eliminated in any case.
                         if not last_trace.getUsageCount():
                             if not last_trace.getPrevious().isUnassignedTrace():
@@ -573,14 +575,20 @@ class StatementDelVariable(StatementBase):
                     % (self.getVariableName(),),
                 )
             else:
-                assert self.variable.isLocalVariable(), self.variable
-
-                result = makeRaiseExceptionReplacementStatement(
-                    statement=self,
-                    exception_type="UnboundLocalError",
-                    exception_value="""local variable '%s' referenced before assignment"""
-                    % variable.getName(),
-                )
+                if self.variable.isLocalVariable():
+                    result = makeRaiseExceptionReplacementStatement(
+                        statement=self,
+                        exception_type="UnboundLocalError",
+                        exception_value="""local variable '%s' referenced before assignment"""
+                        % variable.getName(),
+                    )
+                else:
+                    result = makeRaiseExceptionReplacementStatement(
+                        statement=self,
+                        exception_type="NameError",
+                        exception_value="""name '%s' is not defined"""
+                        % variable.getName(),
+                    )
 
                 return trace_collection.computedStatementResult(
                     result,
@@ -623,8 +631,6 @@ class StatementDelVariable(StatementBase):
             variable=variable, version=self.variable_version, del_node=self
         )
 
-        trace_collection.onVariableContentEscapes(variable)
-
         # Any code could be run, note that.
         trace_collection.onControlFlowEscape(self)
 
@@ -729,7 +735,7 @@ class StatementReleaseVariable(StatementBase):
                 "Uninitialized %s is not released." % (self.variable.getDescription()),
             )
 
-        trace_collection.onVariableContentEscapes(self.variable)
+        # TODO: Annotate value content as escaped, as destruction might run.
 
         # Any code could be run, note that.
         trace_collection.onControlFlowEscape(self)
