@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -32,6 +32,11 @@ from .OutlineNodes import ExpressionOutlineFunctionBase
 class ExpressionClassBody(MarkNeedsAnnotationsMixin, ExpressionOutlineFunctionBase):
     kind = "EXPRESSION_CLASS_BODY"
 
+    __slots__ = ("needs_annotations_dict", "doc")
+
+    if python_version >= 0x340:
+        __slots__ += ("qualname_setup",)
+
     def __init__(self, provider, name, doc, source_ref):
         ExpressionOutlineFunctionBase.__init__(
             self,
@@ -47,7 +52,7 @@ class ExpressionClassBody(MarkNeedsAnnotationsMixin, ExpressionOutlineFunctionBa
         self.doc = doc
 
         # Force creation with proper type.
-        if python_version >= 300:
+        if python_version >= 0x300:
             locals_kind = "python3_class"
         else:
             locals_kind = "python2_class"
@@ -58,6 +63,9 @@ class ExpressionClassBody(MarkNeedsAnnotationsMixin, ExpressionOutlineFunctionBa
             self,
         )
 
+        if python_version >= 0x340:
+            self.qualname_setup = None
+
     def getDetails(self):
         return {
             "name": self.getFunctionName(),
@@ -96,7 +104,7 @@ class ExpressionClassBody(MarkNeedsAnnotationsMixin, ExpressionOutlineFunctionBa
         # they provide "__class__" but nothing else.
 
         if variable_name == "__class__":
-            if python_version < 300:
+            if python_version < 0x300:
                 return self.provider.getVariableForClosure("__class__")
             else:
                 return ExpressionOutlineFunctionBase.getVariableForClosure(
@@ -116,7 +124,7 @@ class ExpressionClassBody(MarkNeedsAnnotationsMixin, ExpressionOutlineFunctionBa
         return False
 
     def mayRaiseException(self, exception_type):
-        return self.getBody().mayRaiseException(exception_type)
+        return self.subnode_body.mayRaiseException(exception_type)
 
     def isUnoptimized(self):
         # Classes all are that.
@@ -127,8 +135,6 @@ class ExpressionSelectMetaclass(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_SELECT_METACLASS"
 
     named_children = ("metaclass", "bases")
-    getMetaclass = ExpressionChildrenHavingBase.childGetter("metaclass")
-    getBases = ExpressionChildrenHavingBase.childGetter("bases")
 
     def __init__(self, metaclass, bases, source_ref):
         ExpressionChildrenHavingBase.__init__(
@@ -144,9 +150,6 @@ class ExpressionBuiltinType3(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_TYPE3"
 
     named_children = ("type_name", "bases", "dict")
-    getTypeName = ExpressionChildrenHavingBase.childGetter("type_name")
-    getBases = ExpressionChildrenHavingBase.childGetter("bases")
-    getDict = ExpressionChildrenHavingBase.childGetter("dict")
 
     def __init__(self, type_name, bases, type_dict, source_ref):
         ExpressionChildrenHavingBase.__init__(
@@ -155,7 +158,46 @@ class ExpressionBuiltinType3(ExpressionChildrenHavingBase):
             source_ref=source_ref,
         )
 
+    def _calculateMetaClass(self):
+        # TODO: Share code with ExpressionSelectMetaclass
+
+        if not self.subnode_bases.isCompileTimeConstant():
+            return None
+
+        # TODO: Want to cache this result probably for speed reasons and it may also
+        # contain allocations for dataclasses, generics, etc.
+
+        # Need to use private CPython API unless we want to re-implement it, pylint: disable=protected-access
+        import ctypes
+
+        ctypes.pythonapi._PyType_CalculateMetaclass.argtypes = [
+            ctypes.py_object,
+            ctypes.py_object,
+        ]
+        ctypes.pythonapi._PyType_CalculateMetaclass.restype = ctypes.py_object
+
+        bases = self.subnode_bases.getCompileTimeConstant()
+
+        return ctypes.pythonapi._PyType_CalculateMetaclass(type, bases)
+
+    def mayRaiseException(self, exception_type):
+        # TODO: In many cases, this will not raise for compile time knowable
+        # case classes. We might ask the bases for the metaclass selected by
+        # compile time inspection.
+        return True
+
     def computeExpression(self, trace_collection):
-        # TODO: Should be compile time computable if bases and dict are.
+
+        # TODO: Can use this to specialize to the correct metaclass at compile
+        # time.
+        # metacls = self._calculateMetaClass()
+
+        # TODO: Should be compile time computable if bases and dict are
+        # allowing that to happen into a dedicated class creation node,
+        # with known metaclass selection.
+
+        # Any exception may be raised.
+        if self.mayRaiseException(BaseException):
+            trace_collection.onExceptionRaiseExit(BaseException)
 
         return self, None, None
