@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -182,14 +182,16 @@ class ExpressionBuiltinLocalsCopy(ExpressionBuiltinLocalsBase):
 
         # Locals is sorted of course.
         def _sorted(pairs):
-            # TODO: Should use a locals scope for this.
-            names = self.getParentVariableProvider().getLocalVariableNames()
+            names = [
+                variable.getName()
+                for variable in self.locals_scope.getProvidedVariables()
+            ]
 
             return tuple(
                 sorted(
                     pairs,
                     key=lambda pair: names.index(
-                        pair.getKey().getCompileTimeConstant()
+                        pair.subnode_key.getCompileTimeConstant()
                     ),
                 )
             )
