@@ -39,7 +39,6 @@ from nuitka.Tracing import (
     memory_logger,
     optimization_logger,
     progress_logger,
-    recursion_logger,
 )
 from nuitka.utils.MemoryUsage import (
     MemoryWatch,
@@ -55,20 +54,6 @@ _progress = Options.isShowProgress()
 _is_verbose = Options.isVerbose()
 
 
-def _attemptRecursion(module):
-    new_modules = module.attemptRecursion()
-
-    if Options.isShowInclusion():
-        for new_module in new_modules:
-            recursion_logger.info(
-                "{source_ref} : {tags} : {message}".format(
-                    source_ref=new_module.getSourceReference().getAsString(),
-                    tags="new_code",
-                    message="Recursed to module package.",
-                )
-            )
-
-
 tag_set = None
 
 
@@ -118,6 +103,8 @@ def optimizeCompiledPythonModule(module):
                 scopes_were_incomplete = module.computeModule()
         except NuitkaForbiddenImportEncounter as e:
             optimization_logger.sysexit("Error, forbidden import '%s' encountered." % e)
+        except SystemExit:
+            raise
         except BaseException:
             general.info("Interrupted while working on '%s'." % module)
             raise
@@ -202,7 +189,7 @@ def optimizeUncompiledPythonModule(module):
 
 def optimizeShlibModule(module):
     # Pick up parent package if any.
-    _attemptRecursion(module)
+    module.attemptRecursion()
 
     Plugins.considerImplicitImports(module=module, signal_change=signalChange)
 
@@ -230,7 +217,7 @@ last_total = 0
 
 
 def _restartProgress():
-    global pass_count, last_total  # Singleton, pylint: disable=global-statement
+    global pass_count  # Singleton, pylint: disable=global-statement
 
     closeProgressBar()
     pass_count += 1
@@ -370,8 +357,6 @@ def optimize(output_filename):
         ):
             demoteCompiledModuleToBytecode(module)
 
-    global pass_count  # Singleton, pylint: disable=global-statement
-
     # Second, "endless" pass.
     while not finished:
         finished = makeOptimizationPass()
