@@ -50,6 +50,8 @@ from .ValueTraces import (
     ValueTraceDeleted,
     ValueTraceEscaped,
     ValueTraceInit,
+    ValueTraceInitStarArgs,
+    ValueTraceInitStarDict,
     ValueTraceLoopComplete,
     ValueTraceLoopIncomplete,
     ValueTraceMerge,
@@ -233,6 +235,20 @@ class CollectionStartpointMixin(CollectionUpdateMixin):
 
         return trace
 
+    def _initVariableInitStarArgs(self, variable):
+        trace = ValueTraceInitStarArgs(self.owner)
+
+        self.addVariableTrace(variable, 0, trace)
+
+        return trace
+
+    def _initVariableInitStarDict(self, variable):
+        trace = ValueTraceInitStarDict(self.owner)
+
+        self.addVariableTrace(variable, 0, trace)
+
+        return trace
+
     def _initVariableUninit(self, variable):
         trace = ValueTraceUninit(owner=self.owner, previous=None)
 
@@ -273,6 +289,7 @@ class CollectionStartpointMixin(CollectionUpdateMixin):
 
     def initVariable(self, variable):
         if variable.isParameterVariable():
+            # TODO: That's not happening, maybe just assert against it.
             result = self._initVariableInit(variable)
         elif variable.isLocalVariable():
             result = self._initVariableUninit(variable)
@@ -373,7 +390,7 @@ class TraceCollectionBase(object):
     def markActiveVariableAsEscaped(self, variable):
         current = self.getVariableCurrentTrace(variable=variable)
 
-        if not current.isUnknownTrace():
+        if not current.isEscapeOrUnknownTrace():
             version = variable.allocateTargetNumber()
 
             self.addVariableTrace(
@@ -384,6 +401,20 @@ class TraceCollectionBase(object):
 
             self.markCurrentVariableTrace(variable, version)
 
+    def markActiveVariableAsUnknown(self, variable):
+        current = self.getVariableCurrentTrace(variable=variable)
+
+        if not current.isUnknownTrace():
+            version = variable.allocateTargetNumber()
+
+            self.addVariableTrace(
+                variable,
+                version,
+                ValueTraceUnknown(owner=self.owner, previous=current),
+            )
+
+            self.markCurrentVariableTrace(variable, version)
+
     def markActiveVariableAsLoopMerge(
         self, loop_node, current, variable, shapes, incomplete
     ):
@@ -406,13 +437,20 @@ class TraceCollectionBase(object):
 
         return result
 
-    def markActiveVariablesAsUnknown(self):
+    def markActiveVariablesAsEscaped(self):
         for variable in self.getActiveVariables():
             if variable.isTempVariable():
                 continue
 
             self.markActiveVariableAsEscaped(variable)
 
+    def markActiveVariablesAsUnknown(self):
+        for variable in self.getActiveVariables():
+            if variable.isTempVariable():
+                continue
+
+            self.markActiveVariableAsUnknown(variable)
+
     @staticmethod
     def signalChange(tags, source_ref, message):
         # This is monkey patched from another module. pylint: disable=I0021,not-callable
@@ -455,18 +493,26 @@ class TraceCollectionBase(object):
         # to be considered escaped, pylint: disable=unused-argument
 
         for variable in self.getActiveVariables():
+            # TODO: Move this to the variable, and prepare and cache it better for
+            # compile time savings.
             if variable.isModuleVariable():
-                # print variable
-
-                self.markActiveVariableAsEscaped(variable)
+                self.markActiveVariableAsUnknown(variable)
 
             elif variable.isLocalVariable():
-                if variable.hasAccessesOutsideOf(self.owner) is not False:
+                if (
+                    str is not bytes
+                    and variable.hasWritersOutsideOf(self.owner) is not False
+                ):
+                    self.markActiveVariableAsUnknown(variable)
+                elif variable.hasAccessesOutsideOf(self.owner) is not False:
                     self.markActiveVariableAsEscaped(variable)
 
     def removeKnowledge(self, node):
         if node.isExpressionVariableRef():
-            self.markActiveVariableAsEscaped(node.variable)
+            if node.variable.isModuleVariable():
+                self.markActiveVariableAsUnknown(node.variable)
+            else:
+                self.markActiveVariableAsEscaped(node.variable)
 
     def onValueEscapeStr(self, node):
         # TODO: We can ignore these for now.
@@ -494,6 +540,8 @@ class TraceCollectionBase(object):
         # remember the delete of the current
         old_trace = self.getVariableCurrentTrace(variable)
 
+        # TODO: Annotate value content as escaped.
+
         variable_trace = ValueTraceDeleted(
             owner=self.owner, del_node=del_node, previous=old_trace
         )
@@ -523,7 +571,10 @@ class TraceCollectionBase(object):
         return result
 
     def onVariableContentEscapes(self, variable):
-        self.markActiveVariableAsEscaped(variable)
+        if variable.isModuleVariable():
+            self.markActiveVariableAsUnknown(variable)
+        else:
+            self.markActiveVariableAsEscaped(variable)
 
     def onExpression(self, expression, allow_none=False):
         if expression is None and allow_none:
@@ -617,6 +668,30 @@ class TraceCollectionBase(object):
         else:
             return expression, change_tags, change_desc
 
+    def computedExpressionResultRaw(self, expression, change_tags, change_desc):
+        """Make sure the replacement expression is computed.
+
+        Use this when a replacement expression needs to be seen by the trace
+        collection and be computed, without causing any duplication, but where
+        otherwise there would be loss of annotated effects.
+
+        This may e.g. be true for nodes that need an initial run to know their
+        exception result and type shape.
+
+        This is for raw, i.e. subnodes are not yet computed automatically.
+        """
+
+        # Need to compute the replacement still.
+        new_expression = expression.computeExpressionRaw(self)
+
+        if new_expression[0] is not expression:
+            # Signal intermediate result as well.
+            self.signalChange(change_tags, expression.getSourceReference(), change_desc)
+
+            return new_expression
+        else:
+            return expression, change_tags, change_desc
+
     def mergeBranches(self, collection_yes, collection_no):
         """Merge two alternative branches into this trace.
 
@@ -882,10 +957,22 @@ class TraceCollectionFunction(CollectionStartpointMixin, TraceCollectionBase):
         )
 
         if function_body.isExpressionFunctionBody():
-            for parameter_variable in function_body.getParameters().getAllVariables():
+            parameters = function_body.getParameters()
+
+            for parameter_variable in parameters.getTopLevelVariables():
                 self._initVariableInit(parameter_variable)
                 self.variable_actives[parameter_variable] = 0
 
+            list_star_variable = parameters.getListStarArgVariable()
+            if list_star_variable is not None:
+                self._initVariableInitStarArgs(list_star_variable)
+                self.variable_actives[list_star_variable] = 0
+
+            dict_star_variable = parameters.getDictStarArgVariable()
+            if dict_star_variable is not None:
+                self._initVariableInitStarDict(dict_star_variable)
+                self.variable_actives[dict_star_variable] = 0
+
         for closure_variable in function_body.getClosureVariables():
             self.initVariableUnknown(closure_variable)
             self.variable_actives[closure_variable] = 0
@@ -955,7 +1042,7 @@ class TraceCollectionModule(CollectionStartpointMixin, TraceCollectionBase):
 def areEmptyTraces(variable_traces):
     """Do these traces contain any writes or accesses."""
     # Many cases immediately return, that is how we do it here,
-    # pylint: disable=too-many-return-statements
+    # pylint: disable=too-many-branches,too-many-return-statements
 
     for variable_trace in variable_traces:
         if variable_trace.isAssignTrace():
@@ -977,6 +1064,11 @@ def areEmptyTraces(variable_traces):
                 # Checking definite is enough, the merges, we shall see
                 # them as well.
                 return False
+        elif variable_trace.isEscapeTrace():
+            if variable_trace.getUsageCount():
+                # Checking definite is enough, the merges, we shall see
+                # them as well.
+                return False
         elif variable_trace.isMergeTrace():
             if variable_trace.getUsageCount():
                 # Checking definite is enough, the merges, we shall see
@@ -1008,6 +1100,8 @@ def areReadOnlyTraces(variable_traces):
             pass
         elif variable_trace.isUnknownTrace():
             return False
+        elif variable_trace.isEscapeTrace():
+            pass
         elif variable_trace.isMergeTrace():
             pass
         elif variable_trace.isLoopTrace():
