@@ -109,12 +109,17 @@ not exist, a list of all available will be given.""",
                         "mediaservice",
                         "printsupport",
                         "platforms",
+                        "platformthemes",
                         "styles",
                     )
                     if self.hasPluginFamily(family)
                 )
             )
 
+            # OpenGL rendering, maybe should be something separate.
+            if self.hasPluginFamily("xcbglintegrations"):
+                self.qt_plugins.add("xcbglintegrations")
+
             self.qt_plugins.remove("sensible")
 
             # Make sure the above didn't detect nothing, which would be
@@ -129,13 +134,13 @@ not exist, a list of all available will be given.""",
             return template % {"binding_name": self.binding_name}
 
         setup_codes = applyBindingName(
-            r"""\
+            r"""
 import os
 import %(binding_name)s.QtCore
 """
         )
 
-        return self.queryRuntimeInformationMultiple(
+        info = self.queryRuntimeInformationMultiple(
             info_name=applyBindingName("%(binding_name)s_info"),
             setup_codes=setup_codes,
             values=(
@@ -172,7 +177,6 @@ import %(binding_name)s.QtCore
                     ),
                 ),
                 (
-                    # TODO: Expose this as an option to add it.
                     "translations_path",
                     applyBindingName(
                         """\
@@ -182,6 +186,11 @@ import %(binding_name)s.QtCore
             ),
         )
 
+        if info is None:
+            self.sysexit("Error, it seems '%s' is not installed." % self.binding_name)
+
+        return info
+
     def _getBindingVersion(self):
         """Get the version of the binding in tuple digit form, e.g. (6,0,3)"""
         return self._getQtInformation().version
@@ -190,6 +199,10 @@ import %(binding_name)s.QtCore
         """Does it include the Nuitka patch, i.e. is a self-built one with it applied."""
         return self._getQtInformation().nuitka_patch_level
 
+    def _getTranslationsPath(self):
+        """Get the path to the Qt translations."""
+        return self._getQtInformation().translations_path
+
     def getQtPluginDirs(self):
         if self.qt_plugins_dirs is not None:
             return self.qt_plugins_dirs
@@ -429,7 +442,7 @@ import %(binding_name)s.QtCore
         if child_name in ("QtQuick", "QtQuickWidgets"):
             yield self._getChildNamed("QtQml")
 
-        if child_name in ("QtQuickWidgets", "QtQml"):
+        if child_name in ("QtQuickWidgets", "QtQml", "QtQuickControls2"):
             yield self._getChildNamed("QtQuick")
 
         if child_name == "Qt":
@@ -556,7 +569,8 @@ if not path.startswith(__nuitka_binary_dir):
         if full_name == self.binding_name:
             if not self.getQtPluginDirs():
                 self.sysexit(
-                    "Error, failed to detect %r plugin directories." % self.binding_name
+                    "Error, failed to detect '%s' plugin directories."
+                    % self.binding_name
                 )
 
             target_plugin_dir = os.path.join(dist_dir, full_name.asPath(), "qt-plugins")
@@ -665,7 +679,7 @@ if not path.startswith(__nuitka_binary_dir):
             plugin_parent = os.path.dirname(self.getQtPluginDirs()[0])
 
             if isWin32Windows():
-                bin_dir = os.path.join(plugin_parent, "bin")
+                bin_dir = plugin_parent
             else:  # TODO verify this for non-Windows!
                 bin_dir = os.path.join(plugin_parent, "libexec")
             target_bin_dir = os.path.join(dist_dir)
@@ -678,16 +692,16 @@ if not path.startswith(__nuitka_binary_dir):
             for f in os.listdir(resources_dir):
                 shutil.copy(os.path.join(resources_dir, f), target_resources_dir)
 
-            translations_dir = os.path.join(plugin_parent, "translations")
-            pos = len(translations_dir) + 1
-            target_translations_dir = os.path.join(
+            translations_path = self._getTranslationsPath()
+            pos = len(translations_path) + 1
+            translations_path = os.path.join(
                 dist_dir,
                 full_name.getTopLevelPackageName().asPath(),
                 "Qt",
                 "translations",
             )
-            for f in getFileList(translations_dir):
-                tar_f = os.path.join(target_translations_dir, f[pos:])
+            for f in getFileList(translations_path):
+                tar_f = os.path.join(translations_path, f[pos:])
                 makePath(os.path.dirname(tar_f))
                 shutil.copyfile(f, tar_f)
 
@@ -837,7 +851,7 @@ This PySide2 version only partially supported through workarounds, full support:
 
         result = NuitkaPluginQtBindingsPluginBase.createPostModuleLoadCode(self, module)
         if result:
-            return result
+            yield result
 
         if (
             self._getNuitkaPatchLevel() < 1
@@ -918,7 +932,7 @@ import PySide2.QtCore
 PySide2.QtCore.QAbstractItemModel.__init_subclass__ = my_init_subclass
 PySide2.QtCore.QObject.__init_subclass__ = my_init_subclass
 """
-            return (
+            yield (
                 code,
                 """\
 Monkey patching classes derived from PySide2 base classes to pass PySide2 checks.""",
