@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -47,7 +47,7 @@ def formatOutput(e):
         reason = e.msg
 
     if colno is not None and (
-        not e.__class__ is IndentationError or python_version < 390
+        not e.__class__ is IndentationError or python_version < 0x390
     ):
         colno = colno - len(message) + len(message.lstrip())
 
@@ -121,7 +121,7 @@ def raiseSyntaxError(reason, source_ref, display_file=True, display_line=True):
         else:
             source_line = None
 
-        raise SyntaxError(
+        exc = SyntaxError(
             reason,
             (
                 source_ref.getFilename(),
@@ -131,4 +131,8 @@ def raiseSyntaxError(reason, source_ref, display_file=True, display_line=True):
             ),
         )
 
-    raise SyntaxError(reason, (None, None, None, None))
+    exc = SyntaxError(reason, (None, None, None, None))
+
+    exc.generated_by_nuitka = True
+
+    raise exc
