@@ -34,10 +34,11 @@ import time
 from contextlib import contextmanager
 
 from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
+    WindowsError,
     basestring,
 )
 from nuitka.PythonVersions import python_version
-from nuitka.Tracing import my_print
+from nuitka.Tracing import my_print, options_logger
 
 from .Importing import importFromInlineCopy
 from .ThreadedExecutor import RLock, getThreadIdent
@@ -138,6 +139,15 @@ def relpath(path, start="."):
         raise
 
 
+def isRelativePath(path):
+    if os.path.isabs(path):
+        return False
+    if path.startswith((".." + os.path.sep, "../")):
+        return False
+
+    return True
+
+
 def makePath(path):
     """Create a directory if it doesn't exist.
 
@@ -155,6 +165,11 @@ def makePath(path):
             os.makedirs(path)
 
 
+def isPathExecutable(path):
+    """Is the given path executable."""
+    return os.path.isfile(path) and os.access(path, os.X_OK)
+
+
 def _getRealPathWindows(path):
     # Slow, because we are using an external process, we it's only for standalone and Python2,
     # which is slow already.
@@ -345,13 +360,23 @@ def deleteFile(path, must_exist):
 
 def splitPath(path):
     """Split path, skipping empty elements."""
-    return tuple(element for element in os.path.split(path) if element)
+    return tuple(
+        element for element in os.path.split(path.rstrip(os.path.sep)) if element
+    )
+
+
+def getFilenameExtension(path):
+    """Get the filename extension.
+
+    Note: For checks on extension, use hasFilenameExtension instead.
+    """
+    return os.path.splitext(os.path.normcase(path))[1]
 
 
 def hasFilenameExtension(path, extensions):
     """Has a filename one of the given extensions."""
 
-    extension = os.path.splitext(os.path.normcase(path))[1]
+    extension = getFilenameExtension(path)
 
     return extension in extensions
 
@@ -442,7 +467,7 @@ def putTextFileContents(filename, contents, encoding=None):
 
     def _writeContents(output_file):
         if isinstance(contents, basestring):
-            print(contents, file=output_file)
+            print(contents, file=output_file, end="")
         else:
             for line in contents:
                 print(line, file=output_file)
@@ -508,7 +533,7 @@ def copyTree(source_path, dest_path):
         dest_path: where to copy to, may already exist
 
     Notes:
-        This must be used over `shutil.copytree` which as troubles
+        This must be used over `shutil.copytree` which has troubles
         with existing directories.
     """
 
@@ -544,6 +569,7 @@ def getWindowsDrive(path):
 
 
 def isPathBelow(path, filename):
+    """Is a path inside of a given directory path."""
     path = os.path.abspath(path)
     filename = os.path.abspath(filename)
 
@@ -554,6 +580,11 @@ def isPathBelow(path, filename):
     return os.path.relpath(filename, path).split(os.path.sep)[0] != ".."
 
 
+def isPathBelowOrSameAs(path, filename):
+    """Is a path inside of a given directory path or the same path as that directory."""
+    return isPathBelow(path, filename) or areSamePaths(path, filename)
+
+
 def getWindowsShortPathName(filename):
     """Gets the short path name of a given long path.
 
@@ -577,7 +608,9 @@ def getWindowsShortPathName(filename):
     output_buf_size = 0
     while True:
         output_buf = ctypes.create_unicode_buffer(output_buf_size)
-        needed = GetShortPathNameW(filename, output_buf, output_buf_size)
+        needed = GetShortPathNameW(
+            os.path.abspath(filename), output_buf, output_buf_size
+        )
 
         if needed == 0:
             # Windows only code, pylint: disable=I0021,undefined-variable
@@ -671,7 +704,22 @@ def resolveShellPatternToFilenames(pattern):
         list - filenames that matched.
     """
 
-    result = glob.glob(pattern)
+    if "**" in pattern:
+        if python_version >= 0x350:
+            result = glob.glob(pattern, recursive=True)
+        else:
+            glob2 = importFromInlineCopy("glob2", must_exist=True)
+
+            if glob2 is None:
+                options_logger.sysexit(
+                    "Using pattern with ** is not supported before Python 3.5 unless glob2 is installed."
+                )
+
+            result = glob2.glob(pattern)
+    else:
+        result = glob.glob(pattern)
+
+    result = [os.path.normpath(filename) for filename in result]
     result.sort()
     return result
 
