@@ -22,19 +22,18 @@ what library to use for what Python. We have an inline copy of PyYAML that
 still does 2.6, on other Pythons, we expect the system to have it installed.
 """
 
-from nuitka.PythonVersions import python_version
-
 from .Importing import importFromInlineCopy
 
 
 def parseYaml(data):
-    if python_version < 0x270:
-        yaml = importFromInlineCopy("yaml", must_exist=False)
-        return yaml.load(data)
-    else:
-        try:
-            import yaml
-        except ImportError:
-            yaml = importFromInlineCopy("yaml", must_exist=False)
+    try:
+        import yaml
+    except ImportError:
+        yaml = importFromInlineCopy("yaml", must_exist=True)
 
-        return yaml.safe_load(data)
+    try:
+        yaml_load_function = yaml.safe_load
+    except AttributeError:
+        yaml_load_function = yaml.load
+
+    return yaml_load_function(data)
